# Select two identical faces shaped like a triangle
# Apply a distorted texture to one of them


# This method samples UV data from each vertex.
# For triangles we end up with only 3 samples, which is not enough for distorted textures.
def sample_vertex
	model = Sketchup.active_model
	sel = model.selection
	tw = Sketchup.create_texture_writer
	
	if sel[0].material.nil?
		source = sel[1] 
		target = sel[0] 
	else
		source = sel[0] 
		target = sel[1] 
	end
	
	t = source.material.texture
	
	model.start_operation('test')
	
	uvh = source.get_UVHelper(true, true, tw)
	
	real_uv = []
	source.vertices.each { |v|
		uvq = uvh.get_front_UVQ(v.position)
		# Correct the buggy UV data from SU
		real_uv << Geom::Point3d.new(uvq.x / uvq.z, uvq.y / uvq.z, 1.0)
	}
	points = target.vertices.collect { |v| v.position }
	uvs = points.zip(real_uv).flatten
	puts uvs.inspect
	target.position_material(source.material, uvs, true)
	
	model.commit_operation
end


# This method samples four UV points from the face's plane
def sample_plane
	model = Sketchup.active_model
	sel = model.selection
	tw = Sketchup.create_texture_writer
	
	if sel[0].material.nil?
		source = sel[1] 
		target = sel[0] 
	else
		source = sel[0] 
		target = sel[1] 
	end
	
	t = source.material.texture
	
	model.start_operation('test')
	
	uvh = source.get_UVHelper(true, true, tw)
	
	sample_points = get_sample_points(source)
	
	real_uv = []
	sample_points.each { |p|
		uvq = uvh.get_front_UVQ(p)
		# Correct the buggy UV data from SU
		real_uv << Geom::Point3d.new(uvq.x / uvq.z, uvq.y / uvq.z, 1.0)
	}
	points = get_sample_points(target)
	uvs = points.zip(real_uv).flatten
	puts uvs.inspect
	target.position_material(source.material, uvs, true)
	
	model.commit_operation
end

def get_sample_points(face)
	# Sample four points from teh face's plane.
	# Start with one of the vertices to pick the first point.
	# From there, create the last 3 sample points by offsetting in x,xy,y direction
	points = []
	points << face.vertices.first.position
	points << points.first.offset([10,0,0]) # x
	points << points.last.offset([0,10,0]) # x,y
	points << points.first.offset([0,10,0]) # y
	return points
end